//+------------------------------------------------------------------+
//|                                                  BarsManager.mqh |
//|                                                   Baruch Fishman |
//|                                         fishman.baruch@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Baruch Fishman"
#property link      "fishman.baruch@gmail.com"
#property version   "1.00"

#include <Arrays/ArrayObj.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class BarsData : CObject
{
private:
   string smbl;
   ENUM_TIMEFRAMES tf;
   datetime lastTickTime;
   int lastBarsCt;
public:
   BarsData(string _smbl, ENUM_TIMEFRAMES _tf);
   BarsData(const BarsData&);
   ~BarsData(void);
   string GetSmbl();
   ENUM_TIMEFRAMES GetTf();
   int GetBarsCt();
};

BarsData::~BarsData(void){}

int BarsData::GetBarsCt()
{
   MqlTick tick;
   if(!SymbolInfoTick(smbl,tick))return lastBarsCt;
   
   //we change to struct and then back to remove seconds from time
   MqlDateTime tickTimeStruct;
   TimeToStruct(tick.time,tickTimeStruct);
   tickTimeStruct.sec = 0;
   tick.time = StructToTime(tickTimeStruct);
   
   if(tick.time == lastTickTime)return lastBarsCt;
   //we get new bars but just in case it needs to be at least 0
   int newBars = MathMax(Bars(smbl,tf,tick.time,lastTickTime) - 1,0);
   lastBarsCt =  newBars + lastBarsCt;
   lastTickTime = tick.time;   
   return lastBarsCt;
}

string BarsData::GetSmbl(void)
{
   return smbl;
}

ENUM_TIMEFRAMES BarsData::GetTf(void)
{
   return tf;
}

BarsData::BarsData(const BarsData &BarsDataIn)
{
   smbl = BarsDataIn.smbl;
   tf = BarsDataIn.tf; 
   lastTickTime = 0;
}

BarsData::BarsData(string _smbl,ENUM_TIMEFRAMES _tf)
{
   smbl = _smbl;
   tf = _tf;
   lastTickTime = 0;
}

class BarsManager
{
private:
   static CArrayObj BarsSmblsTfs;
   static BarsData* GetBarsData(string smbl, ENUM_TIMEFRAMES tf);
   
public:
   BarsManager();
   ~BarsManager();
   static int GetBars(string smbl, ENUM_TIMEFRAMES tf);                    
};

CArrayObj BarsManager::BarsSmblsTfs;

BarsManager::BarsManager(void)
{
   
}

BarsManager::~BarsManager(void)
{
   for(int i=0;i<BarsSmblsTfs.Total();i++)
   {
      delete BarsSmblsTfs.At(i);
   }
   BarsSmblsTfs.Shutdown();
}

static int BarsManager::GetBars(string smbl,ENUM_TIMEFRAMES tf)
{
   BarsData *data = GetBarsData(smbl,tf);
   return data.GetBarsCt();
}

static BarsData* BarsManager::GetBarsData(string smbl,ENUM_TIMEFRAMES tf)
{
   BarsData *data = NULL;
   for(int i=0;i<BarsManager::BarsSmblsTfs.Total();i++)
   {
      data = (BarsData*)BarsManager::BarsSmblsTfs.At(i);
      if(data.GetSmbl() == smbl && data.GetTf() == tf)return data;
   }
   data = new BarsData(smbl,tf);
   BarsSmblsTfs.Add(data);
   return data;  
}